@@ -29,6 +29,9 @@
 #include "glib.h"
 #include "galias.h"
 
+#define DEBUG_MSG(x) /* */
+/* #define DEBUG_MSG(args) g_message args ; */
+
 
 typedef struct _GRealThreadPool GRealThreadPool;
 
@@ -41,6 +44,8 @@ struct _GRealThreadPool
   gboolean running;
   gboolean immediate;
   gboolean waiting;
+  GCompareDataFunc sort_func;
+  gpointer sort_user_data;
 };
 
 /* The following is just an address to mark the stop order for a
@@ -52,33 +57,54 @@ static const gpointer stop_this_thread_marker = (gpointer) &g_thread_pool_new;
 static GAsyncQueue *unused_thread_queue;
 static gint unused_threads = 0;
 static gint max_unused_threads = 0;
-G_LOCK_DEFINE_STATIC (unused_threads);
+static guint max_idle_time = 0;
+G_LOCK_DEFINE_STATIC (settings);
 
 static GMutex *inform_mutex = NULL;
 static GCond *inform_cond = NULL;
 
-static void     g_thread_pool_free_internal (GRealThreadPool* pool);
-static gpointer g_thread_pool_thread_proxy (gpointer data);
-static void     g_thread_pool_start_thread (GRealThreadPool* pool, 
-					    GError **error);
-static void     g_thread_pool_wakeup_and_stop_all (GRealThreadPool* pool);
+
+static void     g_thread_pool_queue_push_unlocked (GRealThreadPool  *pool,
+						   gpointer          data);
+static void     g_thread_pool_free_internal       (GRealThreadPool  *pool);
+static gpointer g_thread_pool_thread_proxy        (gpointer          data);
+static void     g_thread_pool_start_thread        (GRealThreadPool  *pool,
+						   GError          **error);
+static void     g_thread_pool_wakeup_and_stop_all (GRealThreadPool  *pool);
+
 
 #define g_thread_should_run(pool, len) \
   ((pool)->running || (!(pool)->immediate && (len) > 0))
 
+
+static void
+g_thread_pool_queue_push_unlocked (GRealThreadPool *pool,
+				   gpointer         data)
+{
+  if (pool->sort_func) 
+    g_async_queue_push_sorted_unlocked (pool->queue, 
+					data,
+					pool->sort_func, 
+					pool->sort_user_data);
+  else
+    g_async_queue_push_unlocked (pool->queue, data);
+}
+
 static gpointer 
 g_thread_pool_thread_proxy (gpointer data)
 {
   GRealThreadPool *pool = data;
   gboolean watcher = FALSE;
 
+  DEBUG_MSG(("pool:0x%.8x entering proxy ...\n", (guint)pool));
+
   g_async_queue_lock (pool->queue);
   while (TRUE)
     {
-      gpointer task; 
+      gpointer task = NULL; 
       gboolean goto_global_pool = !pool->pool.exclusive;
       gint len = g_async_queue_length_unlocked (pool->queue);
-      
+
       if (g_thread_should_run (pool, len))
 	{
 	  if (watcher)
@@ -91,11 +117,38 @@ g_thread_pool_thread_proxy (gpointer data)
 	      GTimeVal end_time;
 	      g_get_current_time (&end_time);
 	      g_time_val_add (&end_time, G_USEC_PER_SEC / 2); /* 1/2 second */
+	      DEBUG_MSG(("pool:0x%.8x waiting 1/2 second to pop next item "
+			 "in queue (%d running, %d unprocessed) ...\n", 
+			 (guint)pool, 
+			 pool->num_threads, 
+			 g_async_queue_length_unlocked (pool->queue)));
 	      task = g_async_queue_timed_pop_unlocked (pool->queue, &end_time);
 	    }
+	  else if (g_thread_pool_get_max_idle_time() > 0) 
+	    {
+	      /* We always give a maximum time to pop the next task so
+	       * we know that when we evaluate task further down, that
+	       * it has had the maximum time to get a new task and it
+	       * can die */
+	      GTimeVal end_time;
+	      g_get_current_time (&end_time);
+	      DEBUG_MSG(("pool:0x%.8x waiting %d ms max to pop next item in "
+			 "queue (%d running, %d unprocessed) or exiting ...\n", 
+			 (guint)pool, 
+			 g_thread_pool_get_max_idle_time (), 
+			 pool->num_threads, 
+			 g_async_queue_length_unlocked (pool->queue)));
+	      
+	      g_time_val_add (&end_time, g_thread_pool_get_max_idle_time () * 1000);
+ 	      task = g_async_queue_timed_pop_unlocked (pool->queue, &end_time);
+	    }
 	  else
-	    task = g_async_queue_pop_unlocked (pool->queue);
-	  
+	    {
+	      task = g_async_queue_pop_unlocked (pool->queue);
+	      DEBUG_MSG(("pool:0x%.8x new task:0x%.8x poped from pool queue ...\n", 
+			 (guint)pool, (guint)task));
+	}
+
 	  if (task)
 	    {
 	      watcher = FALSE;
@@ -105,19 +158,49 @@ g_thread_pool_thread_proxy (gpointer data)
 		 * the global pool and just hand the data further to
 		 * the next one waiting in the queue */
 		{
-		  g_async_queue_push_unlocked (pool->queue, task);
+		  DEBUG_MSG(("pool:0x%.8x, task:0x%.8x we have too many threads "
+			     "and max is set, pushing task into queue ...\n", 
+			     (guint)pool, (guint)task));
+		  g_thread_pool_queue_push_unlocked (pool, task);
 		  goto_global_pool = TRUE;
 		}
-	      else if (pool->running || !pool->immediate)
+ 	      else if (pool->running || !pool->immediate)
 		{
 		  g_async_queue_unlock (pool->queue);
+		  DEBUG_MSG(("pool:0x%.8x, task:0x%.8x calling func ...\n", 
+			     (guint)pool, (guint)task));
 		  pool->pool.func (task, pool->pool.user_data);
 		  g_async_queue_lock (pool->queue);
 		}
+	    } 
+ 	  else if (g_thread_pool_get_max_idle_time() > 0)  
+	    {
+	      G_LOCK (settings);
+	      if (pool->num_threads > max_unused_threads) {
+		G_UNLOCK (settings);
+		pool->num_threads--;      
+		
+		DEBUG_MSG(("pool:0x%.8x queue timed pop has no tasks waiting, "
+			   "so stopping thread (%d running, %d unprocessed) ...\n", 
+			   (guint)pool, 
+			   pool->num_threads, 
+			   g_async_queue_length_unlocked (pool->queue)));
+		g_async_queue_unlock (pool->queue);
+		
+		return NULL; 
+	      }
+	      G_UNLOCK (settings);
 	    }
 	  len = g_async_queue_length_unlocked (pool->queue);
 	}
 
+      DEBUG_MSG(("pool:0x%.8x, len:%d, watcher:%s, exclusive:%s, should run:%s\n",
+		 (guint)pool, 
+		 len, 
+		 watcher ? "true" : "false", 
+		 pool->pool.exclusive ? "true" : "false", 
+		 g_thread_should_run (pool, len) ? "true" : "false"));
+
       if (!g_thread_should_run (pool, len))
 	{
 	  g_cond_broadcast (inform_cond);
@@ -128,6 +211,11 @@ g_thread_pool_thread_proxy (gpointer data)
 	  /* At this pool there are no threads waiting, but tasks are. */
 	  goto_global_pool = FALSE; 
 	}
+      else if (len < 1 && g_thread_pool_get_max_idle_time () > 0) 
+	{
+	  goto_global_pool = FALSE;
+	  watcher = FALSE;
+	}
       else if (len == 0 && !watcher && !pool->pool.exclusive)
 	{
 	  /* Here neither threads nor tasks are queued and we didn't
@@ -136,10 +224,11 @@ g_thread_pool_thread_proxy (gpointer data)
 	   * switches. */
 	  goto_global_pool = FALSE;
 	  watcher = TRUE;
-	}
+	} 
 
       if (goto_global_pool)
 	{
+	  DEBUG_MSG(("pool:0x%.8x, now in the global pool\n", (guint)pool));
 	  pool->num_threads--;
 
 	  if (!pool->running && !pool->waiting)
@@ -162,23 +251,27 @@ g_thread_pool_thread_proxy (gpointer data)
 	  
 	  g_async_queue_lock (unused_thread_queue);
 
-	  G_LOCK (unused_threads);
-	  if ((unused_threads >= max_unused_threads && 
+	  G_LOCK (settings);
+	  if ((unused_threads >= max_unused_threads &&
 	       max_unused_threads != -1))
 	    {
-	      G_UNLOCK (unused_threads);
+	      G_UNLOCK (settings);
 	      g_async_queue_unlock (unused_thread_queue);
+	      DEBUG_MSG(("pool:0x%.8x stopping thread (%d running, %d unprocessed) ...\n", 
+			 (guint)pool, 
+			 pool->num_threads, 
+			 g_async_queue_length_unlocked (pool->queue)));
 	      /* Stop this thread */
 	      return NULL;      
 	    }
 	  unused_threads++;
-	  G_UNLOCK (unused_threads);
+	  G_UNLOCK (settings);
 
 	  pool = g_async_queue_pop_unlocked (unused_thread_queue);
 
-	  G_LOCK (unused_threads);
+	  G_LOCK (settings);
 	  unused_threads--;
-	  G_UNLOCK (unused_threads);
+	  G_UNLOCK (settings);
 
 	  g_async_queue_unlock (unused_thread_queue);
 	  
@@ -232,6 +325,8 @@ g_thread_pool_start_thread (GRealThreadPool  *pool,
   /* See comment in g_thread_pool_thread_proxy as to why this is done
    * here and not there */
   pool->num_threads++;
+  DEBUG_MSG(("pool:0x%.8x thread created, (running:%d)\n", 
+	     (guint)pool, pool->num_threads));
 }
 
 /**
@@ -293,6 +388,8 @@ g_thread_pool_new (GFunc            func,
   retval->max_threads = max_threads;
   retval->num_threads = 0;
   retval->running = TRUE;
+  retval->sort_func = NULL;
+  retval->sort_user_data = NULL;
 
   G_LOCK (init);
   
@@ -365,7 +462,7 @@ g_thread_pool_push (GThreadPool     *pool,
     /* No thread is waiting in the queue */
     g_thread_pool_start_thread (real, error);
 
-  g_async_queue_push_unlocked (real->queue, data);
+  g_thread_pool_queue_push_unlocked (real, data);
   g_async_queue_unlock (real->queue);
 }
 
@@ -599,7 +696,7 @@ g_thread_pool_wakeup_and_stop_all (GRealThreadPool* pool)
 
   pool->immediate = TRUE; 
   for (i = 0; i < pool->num_threads; i++)
-    g_async_queue_push_unlocked (pool->queue, GUINT_TO_POINTER (1));
+    g_thread_pool_queue_push_unlocked (pool, GUINT_TO_POINTER (1));
 }
 
 /**
@@ -615,7 +712,7 @@ g_thread_pool_set_max_unused_threads (gint max_threads)
 {
   g_return_if_fail (max_threads >= -1);  
 
-  G_LOCK (unused_threads);
+  G_LOCK (settings);
   
   max_unused_threads = max_threads;
 
@@ -630,7 +727,7 @@ g_thread_pool_set_max_unused_threads (gint max_threads)
       g_async_queue_unlock (unused_thread_queue);
     }
     
-  G_UNLOCK (unused_threads);
+  G_UNLOCK (settings);
 }
 
 /**
@@ -645,9 +742,9 @@ g_thread_pool_get_max_unused_threads (void)
 {
   gint retval;
   
-  G_LOCK (unused_threads);
+  G_LOCK (settings);
   retval = max_unused_threads;
-  G_UNLOCK (unused_threads);
+  G_UNLOCK (settings);
 
   return retval;
 }
@@ -659,13 +756,14 @@ g_thread_pool_get_max_unused_threads (void)
  *
  * Return value: the number of currently unused threads
  **/
-guint g_thread_pool_get_num_unused_threads (void)
+guint 
+g_thread_pool_get_num_unused_threads (void)
 {
   guint retval;
   
-  G_LOCK (unused_threads);
+  G_LOCK (settings);
   retval = unused_threads;
-  G_UNLOCK (unused_threads);
+  G_UNLOCK (settings);
 
   return retval;
 }
@@ -677,12 +775,105 @@ guint g_thread_pool_get_num_unused_threads (void)
  * maximal number of unused threads. This function can be used to
  * regularly stop all unused threads e.g. from g_timeout_add().
  **/
-void g_thread_pool_stop_unused_threads (void)
+void
+g_thread_pool_stop_unused_threads (void)
 { 
   guint oldval = g_thread_pool_get_max_unused_threads ();
   g_thread_pool_set_max_unused_threads (0);
   g_thread_pool_set_max_unused_threads (oldval);
 }
 
+/**
+ * g_thread_pool_set_sort_function:
+ * @pool: a #GThreadPool
+ * @func: the #GCompareDataFunc used to sort the list of tasks. 
+ *     This function is passed two tasks. It should return
+ *     0 if the order in which they are handled does not matter, 
+ *     a negative value if the first task should be processed before
+ *     the second or a positive value if the second task should be 
+ *     processed first.
+ * @user_data: user data passed to @func.
+ *
+ * Sets the function used to sort the list of tasks. This allows the
+ * tasks to be processed by a priority determined by @func, and not
+ * just in the order in which they were added to the pool.
+ *
+ * Since: 2.10
+ **/
+void 
+g_thread_pool_set_sort_function (GThreadPool      *pool,
+				 GCompareDataFunc  func,
+				 gpointer          user_data)
+{ 
+  GRealThreadPool *real = (GRealThreadPool*) pool;
+
+  g_return_if_fail (real);
+  g_return_if_fail (real->running);
+
+  g_async_queue_lock (real->queue);
+
+  real->sort_func = func;
+  real->sort_user_data = user_data;
+  
+  if (func) 
+    g_async_queue_sort_unlocked (real->queue, 
+				 real->sort_func,
+				 real->sort_user_data);
+
+  g_async_queue_unlock (real->queue);
+}
+
+/**
+ * g_thread_pool_set_max_idle_time:
+ * @interval: the maximum @interval (1/1000ths of a second) a thread
+ *     can be idle. 
+ *
+ * This function will set the maximum @interval that a thread waiting
+ * in the pool for new tasks can be idle for before being
+ * stopped. This function is similar to calling
+ * g_thread_pool_stop_unused_threads() on a regular timeout, except,
+ * this is done on a per thread basis.    
+ *
+ * By setting @interval to 0, idle threads will not be stopped.
+ *  
+ * This function makes use of g_async_queue_timed_pop () using
+ * @interval.
+ *
+ * Since: 2.10
+ **/
+void
+g_thread_pool_set_max_idle_time (guint interval)
+{ 
+  G_LOCK (settings);
+  max_idle_time = interval;
+  G_UNLOCK (settings);
+}
+
+/**
+ * g_thread_pool_get_max_idle_time:
+ * 
+ * This function will return the maximum @interval that a thread will
+ * wait in the thread pool for new tasks before being stopped.
+ *
+ * If this function returns 0, threads waiting in the thread pool for
+ * new work are not stopped.
+ *
+ * Return value: the maximum @interval to wait for new tasks in the
+ *     thread pool before stopping the thread (1/1000ths of a second).
+ *  
+ * Since: 2.10
+ **/
+guint
+g_thread_pool_get_max_idle_time (void)
+{ 
+  guint retval;
+
+  G_LOCK (settings);
+  retval = max_idle_time;
+  G_UNLOCK (settings);
+
+  return retval;
+}
+
 #define __G_THREADPOOL_C__
 #include "galiasdef.c"
